﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Interop;

namespace TestProject
{
    /// <summary>
    /// 컨트롤 호스트
    /// </summary>
    public class ControlHost : HwndHost
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 윈도우 생성하기 (확장) - CreateWindowEx(extendedStyle, className, windowName, style, x, y, width, height, parentWindowHandle, menuHandle, instanceHandle, parameter)

        /// <summary>
        /// 윈도우 생성하기 (확장)
        /// </summary>
        /// <param name="extendedStyle">확장 스타일</param>
        /// <param name="className">클래스명</param>
        /// <param name="windowName">윈도우명</param>
        /// <param name="style">스타일</param>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="parentWindowHandle">부모 윈도우 핸들</param>
        /// <param name="menuHandle">메뉴 핸들</param>
        /// <param name="instanceHandle">인스턴스 핸들</param>
        /// <param name="parameter">매개 변수</param>
        /// <returns>윈도우 핸들</returns>
        [DllImport("user32", EntryPoint = "CreateWindowEx", CharSet = CharSet.Unicode)]
        private static extern IntPtr CreateWindowEx
        (
            int    extendedStyle,
            string className,
            string windowName,
            int    style,
            int    x,
            int    y,
            int    width,
            int    height,
            IntPtr parentWindowHandle,
            IntPtr menuHandle,
            IntPtr instanceHandle,
            [MarshalAs(UnmanagedType.AsAny)] object parameter
        );

        #endregion
        #region 윈도우 제거하기 - DestroyWindow(windowHandle)

        /// <summary>
        /// 윈도우 제거하기
        /// </summary>
        /// <param name="windowHandle">윈도우 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("user32", EntryPoint = "DestroyWindow", CharSet = CharSet.Unicode)]
        private static extern bool DestroyWindow(IntPtr windowHandle);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// WS_CHILD
        /// </summary>
        private const int WS_CHILD = 0x40000000;

        /// <summary>
        /// WS_VISIBLE
        /// </summary>
        private const int WS_VISIBLE = 0x10000000;

        /// <summary>
        /// LBS_NOTIFY
        /// </summary>
        private const int LBS_NOTIFY = 0x00000001;

        /// <summary>
        /// HOST_ID
        /// </summary>
        private const int HOST_ID = 0x00000002;

        /// <summary>
        /// LISTBOX_ID
        /// </summary>
        private const int LISTBOX_ID = 0x00000001;

        /// <summary>
        /// WS_VSCROLL
        /// </summary>
        private const int WS_VSCROLL = 0x00200000;

        /// <summary>
        /// WS_BORDER
        /// </summary>
        private const int WS_BORDER = 0x00800000;

        /// <summary>
        /// 호스트 너비
        /// </summary>
        private readonly int hostWidth;

        /// <summary>
        /// 호스트 높이
        /// </summary>
        private readonly int hostHeight;

        /// <summary>
        /// 호스트 윈도우 핸들
        /// </summary>
        private IntPtr hostWindowHandle;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 리스트 박스 윈도우 핸들 - ListBoxWindowHandle

        /// <summary>
        /// 리스트 박스 윈도우 핸들
        /// </summary>
        public IntPtr ListBoxWindowHandle { get; private set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ControlHost(width, height)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        public ControlHost(double width, double height)
        {
            this.hostWidth  = (int)width;
            this.hostHeight = (int)height;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 윈도우 코어 만들기 - BuildWindowCore(parentHandleRef)

        /// <summary>
        /// 윈도우 코어 만들기
        /// </summary>
        /// <param name="parentHandleRef">부모 핸들 참조</param>
        /// <returns>핸들 참조</returns>
        protected override HandleRef BuildWindowCore(HandleRef parentHandleRef)
        {
            ListBoxWindowHandle = IntPtr.Zero;

            this.hostWindowHandle = IntPtr.Zero;

            this.hostWindowHandle = CreateWindowEx
            (
                0,
                "static",
                "",
                WS_CHILD | WS_VISIBLE,
                0,
                0,
                this.hostWidth,
                this.hostHeight,
                parentHandleRef.Handle,
                (IntPtr)HOST_ID,
                IntPtr.Zero,
                0
            );

            ListBoxWindowHandle = CreateWindowEx
            (
                0,
                "listbox",
                "",
                WS_CHILD | WS_VISIBLE | LBS_NOTIFY | WS_VSCROLL | WS_BORDER,
                0,
                0,
                this.hostWidth,
                this.hostHeight,
                this.hostWindowHandle,
                (IntPtr)LISTBOX_ID,
                IntPtr.Zero,
                0
            );

            return new HandleRef(this, this.hostWindowHandle);
        }

        #endregion
        #region 윈도우 프로시저 처리하기 - WndProc(windowHandle, message, wordParameter, longParameter, handled)

        /// <summary>
        /// 윈도우 프로시저 처리하기
        /// </summary>
        /// <param name="windowHandle">윈도우 핸들</param>
        /// <param name="message">메시지</param>
        /// <param name="wordParameter">WORD 매개 변수</param>
        /// <param name="longParameter">LONG 매개 변수</param>
        /// <param name="handled">처리 여부</param>
        /// <returns>핸들</returns>
        protected override IntPtr WndProc(IntPtr windowHandle, int message, IntPtr wordParameter, IntPtr longParameter, ref bool handled)
        {
            handled = false;

            return IntPtr.Zero;
        }

        #endregion
        #region 윈도우 코어 제거하기 - DestroyWindowCore(windowHandleRef)

        /// <summary>
        /// 윈도우 코어 제거하기
        /// </summary>
        /// <param name="windowHandleRef">윈도우 핸들 참조</param>
        protected override void DestroyWindowCore(HandleRef windowHandleRef)
        {
            DestroyWindow(windowHandleRef.Handle);
        }

        #endregion
    }
}