﻿using System;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 메시지 보내기 - SendMessage(windowHandle, message, wordParameter, longParameter)

        /// <summary>
        /// 메시지 보내기
        /// </summary>
        /// <param name="windowHandle">윈도우 핸들</param>
        /// <param name="message">메시지</param>
        /// <param name="wordParameter">WORD 매개 변수</param>
        /// <param name="longParameter">LONG 매개 변수</param>
        /// <returns>처리 결과</returns>
        [DllImport("user32", EntryPoint = "SendMessage", CharSet = CharSet.Unicode)]
        private static extern int SendMessage(IntPtr windowHandle, int message, IntPtr wordParameter, IntPtr longParameter);

        #endregion
        #region 메시지 보내기 - SendMessage(windowHandle, message, wordParameter, longParameter)

        /// <summary>
        /// 메시지 보내기
        /// </summary>
        /// <param name="windowHandle">윈도우 핸들</param>
        /// <param name="message">메시지</param>
        /// <param name="wordParameter">WORD 매개 변수</param>
        /// <param name="longParameter">LONG 매개 변수</param>
        /// <returns>처리 결과</returns>
        [DllImport("user32", EntryPoint = "SendMessage", CharSet = CharSet.Unicode)]
        private static extern int SendMessage(IntPtr windowHandle, int message, int wordParameter, [MarshalAs(UnmanagedType.LPWStr)] StringBuilder longParameter);

        #endregion
        #region 메시지 보내기 - SendMessage(windowHandle, message, wordParameter, longParameter)

        /// <summary>
        /// 메시지 보내기
        /// </summary>
        /// <param name="windowHandle">윈도우 핸들</param>
        /// <param name="message">메시지</param>
        /// <param name="wordParameter">WORD 매개 변수</param>
        /// <param name="longParameter">LONG 매개 변수</param>
        /// <returns>처리 결과</returns>
        [DllImport("user32", EntryPoint = "SendMessage", CharSet = CharSet.Unicode)]
        private static extern IntPtr SendMessage(IntPtr windowHandle, int message, IntPtr wordParameter, string longParameter);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// WM_COMMAND
        /// </summary>
        private const int WM_COMMAND = 0x00000111;

        /// <summary>
        /// LB_SEL_CHANGE
        /// </summary>
        private const int LB_SEL_CHANGE = 0x00000001;

        /// <summary>
        /// LB_GET_CUR_SEL
        /// </summary>
        private const int LB_GET_CUR_SEL = 0x00000188;

        /// <summary>
        /// LB_GET_TEXT_LEN
        /// </summary>
        private const int LB_GET_TEXT_LEN = 0x0000018A;

        /// <summary>
        /// LB_ADD_STRING
        /// </summary>
        private const int LB_ADD_STRING = 0x00000180;

        /// <summary>
        /// LB_GET_TEXT
        /// </summary>
        private const int LB_GET_TEXT = 0x00000189;

        /// <summary>
        /// LB_DELETE_STRING
        /// </summary>
        private const int LB_DELETE_STRING = 0x00000182;

        /// <summary>
        /// LB_GET_COUNT
        /// </summary>
        private const int LB_GET_COUNT = 0x0000018B;

        /// <summary>
        /// 리스트 박스 핸들
        /// </summary>
        private IntPtr listBoxHandle;

        /// <summary>
        /// 항목 수
        /// </summary>
        private int itemCount;

        /// <summary>
        /// 컨트롤 호스트
        /// </summary>
        private ControlHost controlHost;

        /// <summary>
        /// 선택 항목 인덱스
        /// </summary>
        private int selectedItemIndex;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            Loaded                  += Window_Loaded;
            this.appendButton.Click += appendButton_Click;
            this.deleteButton.Click += deleteButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

        /// <summary>
        /// 윈도우 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Loaded(object sender, EventArgs e)
        {
            this.controlHost = new ControlHost(this.hostBorder.ActualWidth, this.hostBorder.ActualHeight);

            this.hostBorder.Child = this.controlHost;

            this.controlHost.MessageHook += controlHost_MessageHook;

            this.listBoxHandle = this.controlHost.ListBoxWindowHandle;

            for(int i = 0; i < 15; i++)
            {
                string itemText = "Item" + i;

                SendMessage(this.listBoxHandle, LB_ADD_STRING, IntPtr.Zero, itemText);
            }

            this.itemCount = SendMessage(this.listBoxHandle, LB_GET_COUNT, IntPtr.Zero, IntPtr.Zero);

            this.itemCountTextBlock.Text = this.itemCount.ToString();
        }

        #endregion
        #region 컨트롤 호스트 메시지 후킹 처리하기 - controlHost_MessageHook(windowHandle, message, wordParameter, longParameter, handled)

        /// <summary>
        /// 컨트롤 호스트 메시지 후킹 처리하기
        /// </summary>
        /// <param name="windowHandle">윈도우 핸들</param>
        /// <param name="message">메시지</param>
        /// <param name="wordParameter">WORD 매개 변수</param>
        /// <param name="longParameter">LONG 매개 변수</param>
        /// <param name="handled">처리 여부</param>
        /// <returns>핸들</returns>
        private IntPtr controlHost_MessageHook(IntPtr windowHandle, int message, IntPtr wordParameter, IntPtr longParameter, ref bool handled)
        {
            int textLength;

            handled = false;

            if(message == WM_COMMAND)
            {
                switch((uint) wordParameter.ToInt32() >> 16 & 0xffff)
                {
                    case LB_SEL_CHANGE :

                        this.selectedItemIndex = SendMessage(this.controlHost.ListBoxWindowHandle, LB_GET_CUR_SEL, IntPtr.Zero, IntPtr.Zero);

                        textLength = SendMessage(this.controlHost.ListBoxWindowHandle, LB_GET_TEXT_LEN, IntPtr.Zero, IntPtr.Zero);

                        StringBuilder stringBuilder = new StringBuilder();

                        SendMessage(this.listBoxHandle, LB_GET_TEXT, this.selectedItemIndex, stringBuilder);

                        this.selectedTextBlock.Text = stringBuilder.ToString();

                        handled = true;

                        break;
                }
            }

            return IntPtr.Zero;
        }

        #endregion
        #region Append 버튼 클릭시 처리하기 - appendButton_Click(sender, e)

        /// <summary>
        /// Append 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void appendButton_Click(object sender, EventArgs e)
        {
            if(this.appendTextBox.Text != string.Empty)
            {
                SendMessage(this.listBoxHandle, LB_ADD_STRING, IntPtr.Zero, this.appendTextBox.Text);
            }

            this.itemCount = SendMessage(this.listBoxHandle, LB_GET_COUNT, IntPtr.Zero, IntPtr.Zero);

            this.itemCountTextBlock.Text = this.itemCount.ToString();
        }

        #endregion
        #region Delete 버튼 클릭시 처리하기 - deleteButton_Click(sender, e)

        /// <summary>
        /// Delete 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void deleteButton_Click(object sender, EventArgs e)
        {
            this.selectedItemIndex = SendMessage(this.controlHost.ListBoxWindowHandle, LB_GET_CUR_SEL, IntPtr.Zero, IntPtr.Zero);

            if(this.selectedItemIndex != -1)
            {
                SendMessage(this.listBoxHandle, LB_DELETE_STRING, (IntPtr) this.selectedItemIndex, IntPtr.Zero);
            }

            this.itemCount = SendMessage(this.listBoxHandle, LB_GET_COUNT, IntPtr.Zero, IntPtr.Zero);

            this.itemCountTextBlock.Text = this.itemCount.ToString();
        }

        #endregion
    }
}